@@ -115,6 +115,7 @@ str(l0 <- Matrix(FALSE, nrow=100, ncol = 200))
 stopifnot(all(!l0),
           identical(FALSE, any(l0)))
 
+if(!interactive()) warnings()
 ## really large {length(<dense equivalent>) is beyond R's limits}:
 op <- options(warn = 2) # warnings here are errors
 n <- 50000L
@@ -140,7 +141,10 @@ dm <- as(m0, "denseMatrix")
 stopifnot(all.equal(rcond(dm), rcond(m.), tol = 1e-14),
 	  ##^^^^^^^ dm and m. are both dense, interestingly small differences
 	  ## show in at least one case of optimized BLAS
-	  all.equal(rcond(dm), 0.4899474520656))
+	  all.equal(rcond(dm), 0.4899474520656),
+          ## show(<dgRMatrix>) had revealed a bug in C:
+          identical(capture.output(show(as(m0, "RsparseMatrix")))[-(1:2)],
+                    gsub("0", ".",  capture.output(show(m.))[-1])))
 rm(m)
 
 ###--  Sparse Triangular :
@@ -731,6 +735,7 @@ as(sv, "zsparseVector")
 stopifnot(identical(sv., sv2),
 	  identical(  Matrix(sv, 3,4, byrow=TRUE),
 		    t(Matrix(sv, 4,3))))
+options(warn = 0)# no longer error
 
 
 ## "Large" sparse:
